// srp.h info


// Client Defines
#define SRP_BUFFER_SIZE 100
#define SRP_RTO 200
#define SRP_NUM_RETRIES 2
#define SRP_LOSS_SMOOTH 0.95
#define SRP_LAT_SMOOTH 0.85
#define SRP_EXP_SMOOTH 0.85
#define SRP_EXPDEV_SMOOTH 0.85
#define SRP_RTT_SMOOTH 0.85
#define SRP_RTTDEV_SMOOTH 0.85
#define SRP_MAX_PROBE_TIME 500
#define SRP_MAX_UNK_RTT 1000

// Server Defines
#define SRP_PORT 55555
#define MAX_BACKLOG 10
#define NUM_FRAMES 10000
#define FRAME_SPEED 160
#define FRAME_SIZE 1280

// Common
#define SRP_MAX_DATA_ID 32000 
#define SRP_MAX_DATA_SIZE 1480
#define SRP_STATUS_MISSING 0
#define SRP_DATA_OK 1
#define SRP_DATA_MISSING 2
#define SRP_TYPE_DATA 0
#define SRP_TYPE_NACK 1
#define SRP_TYPE_PROBE 2
#define SRP_NO_TIME -1

// Error codes to be defined for SRP

#define ENOMEM -1
#define EAGAIN -2
#define ESRPSIZEERROR -3
#define EINVAL -4
#define EDEADLK -5
#define EPERM -6
#define EBADF -7
#define ENOTSOCK -8
#define EFAULT -9
#define EMSGSIZE -10
#define ENOBUFS -11
#define EINTR -12
#define EPIPE -13
#define ESRPNULLPTR -14
#define ESRCH -15


// Common Structures

struct srp_buffer_unit {
  int size;
  unsigned int data_id;
  long arrival;
  long rtt;
  void *data;
};

struct srp_buffer_node {
  struct srp_buffer_unit msg;
  struct srp_buffer_node *next;
};

struct srp_packet_header {
  int srp_type;
  long timestamp;
};

struct srp_data_packet_header {
  unsigned int srp_data_id;
  unsigned int srp_data_size;
};

struct srp_data_packet {
  struct srp_packet_header srp_head;
  struct srp_data_packet_header srp_data_head;
  char data[SRP_MAX_DATA_SIZE];
};

struct srp_nack_packet {
  struct srp_packet_header srp_head;
  char status;
  unsigned int srp_data_id;
};

struct srp_probe_packet {
  struct srp_packet_header srp_head;
};


/* Server Structures */

struct srp_server_socket {
  int socket;
  unsigned int flags;
  struct sockaddr to; 
  int tolen;
  struct srp_buffer_unit *srp_buffer;
  int srp_cur_data_id;
  pthread_t srp_recv_thread;
  pthread_mutex_t mutex;
};


/* Client Structures */

struct srp_client_socket {
  int socketnum;
  unsigned int flags; 
  struct sockaddr to; 
  int tolen;
  struct srp_buffer_node *buffer;
  int expected;
  long int data_pkt_recv;
  int give_up;
  long  req_lat; /* Latency requested by application */
  long  ave_lat; /* Average latency */
  float req_loss; /* Requested loss by application */
  float ave_loss; /* Average Loss */
  long rtt; /* Round trip time */
  long rtt_dev; /* Round trip time deviation */
  long last_rtt; /* Time since last rtt update */
  long last_probe; /* Time since last probe was sent */
  long lnrt; /* Last Normal Reception Time */
  int last_lnrt; /* Number of data ids since last LNRT */
  long exp_interval; /* Expected Interval (how long expected between regular messages) */
  long exp_dev; /* Expected Deviation (how much time has deviated from expected) */
  long retrans_time;  /* When a retransmission was sent */
  char algorithm[10];

};


// Client Prototypes

struct srp_client_socket *srp_client_start(int, unsigned int, const struct sockaddr *, int, float, int);
int srp_recvfrom(struct srp_client_socket *, void *, int);
void send_nack(int, struct srp_client_socket *);
int add_data(struct srp_buffer_node **, int, char *, int, long, long); 
void print_data(struct srp_buffer_node *);
struct srp_buffer_node *check_list(struct srp_buffer_node **, int);
int lessthan(int, int);
int lessthanequal(int, int);
int greaterthan(int, int);
int greaterthanequal(int, int);
int update_exp(struct srp_client_socket *, long);
int update_lat(struct srp_client_socket *);
int update_cache_lat(struct srp_client_socket *, long, long);
int update_loss(struct srp_client_socket *, int);
int update_rtt(struct srp_client_socket *, long, long);
int should_retransmit(struct srp_client_socket *, long);
long rto_time(struct srp_client_socket *, int);
long diftime(long, long);
long gettime(void);
int lessthanequal_time(long, long);
void reset(struct srp_client_socket *, int, long);


// Server Prototypes

struct srp_server_socket *srp_server_start(int, unsigned int, const struct sockaddr *, int);
int srp_sendto(struct srp_server_socket *, const void *, int);
int srp_recv_messages(void *);
void srp_print_buffer(struct srp_server_socket *);
int srp_server_stop(struct srp_server_socket *);














































